@@ -1,23 +1,20 @@
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
 from collections import Mapping
 from contextlib import contextmanager
 import pandas as pd
 
 from . import formatting
 from .utils import Frozen
-from .merge import merge_coords_only, align_and_merge_coords
-from .pycompat import iteritems, basestring, OrderedDict
-from .variable import default_index_coordinate
+from .merge import (
+    merge_coords, expand_and_merge_variables, merge_coords_for_inplace_math)
+from .pycompat import OrderedDict
 
 
 class AbstractCoordinates(Mapping, formatting.ReprMixin):
     def __getitem__(self, key):
-        if (key in self._names or
-            (isinstance(key, basestring) and
-             key.split('.')[0] in self._names)):
-            # allow indexing current coordinates or components
-            return self._data[key]
-        else:
-            raise KeyError(key)
+        raise NotImplementedError
 
     def __setitem__(self, key, value):
         self.update({key: value})
@@ -53,18 +50,41 @@ class AbstractCoordinates(Mapping, formatting.ReprMixin):
         return self._data.dims
 
     def to_index(self, ordered_dims=None):
-        """Convert all index coordinates into a :py:class:`pandas.MultiIndex`
+        """Convert all index coordinates into a :py:class:`pandas.Index`.
+
+        Parameters
+        ----------
+        ordered_dims : sequence, optional
+            Possibly reordered version of this object's dimensions indicating
+            the order in which dimensions should appear on the result.
+
+        Returns
+        -------
+        pandas.Index
+            Index subclass corresponding to the outer-product of all dimension
+            coordinates. This will be a MultiIndex if this object is has more
+            than more dimension.
         """
         if ordered_dims is None:
             ordered_dims = self.dims
-        indexes = [self.variables[k].to_index() for k in ordered_dims]
-        return pd.MultiIndex.from_product(indexes, names=list(ordered_dims))
+        elif set(ordered_dims) != set(self.dims):
+            raise ValueError('ordered_dims must match dims, but does not: '
+                             '{} vs {}'.format(ordered_dims, self.dims))
+
+        if len(ordered_dims) == 0:
+            raise ValueError('no valid index for a 0-dimensional object')
+        elif len(ordered_dims) == 1:
+            (dim,) = ordered_dims
+            return self._data.get_index(dim)
+        else:
+            indexes = [self._data.get_index(k) for k in ordered_dims]
+            names = list(ordered_dims)
+            return pd.MultiIndex.from_product(indexes, names=names)
 
     def update(self, other):
         other_vars = getattr(other, 'variables', other)
-        coords = align_and_merge_coords([self.variables, other_vars],
-                                        priority_arg=1,
-                                        indexes=self.indexes)
+        coords = merge_coords([self.variables, other_vars],
+                              priority_arg=1, indexes=self.indexes)
         self._update_coords(coords)
 
     def _merge_raw(self, other):
@@ -73,7 +93,8 @@ class AbstractCoordinates(Mapping, formatting.ReprMixin):
             variables = OrderedDict(self.variables)
         else:
             # don't align because we already called xarray.align
-            variables = merge_coords_only([self.variables, other.variables])
+            variables = expand_and_merge_variables(
+                [self.variables, other.variables])
         return variables
 
     @contextmanager
@@ -86,7 +107,7 @@ class AbstractCoordinates(Mapping, formatting.ReprMixin):
             # first
             priority_vars = OrderedDict(
                 (k, v) for k, v in self.variables.items() if k not in self.dims)
-            variables = merge_coords_only(
+            variables = merge_coords_for_inplace_math(
                 [self.variables, other.variables], priority_vars=priority_vars)
             yield
             self._update_coords(variables)
@@ -119,7 +140,7 @@ class AbstractCoordinates(Mapping, formatting.ReprMixin):
             return self.to_dataset()
         else:
             other_vars = getattr(other, 'variables', other)
-            coords = merge_coords_only([self.variables, other_vars])
+            coords = expand_and_merge_variables([self.variables, other_vars])
             return Dataset._from_vars_and_coord_names(coords, set(coords))
 
 
@@ -143,6 +164,11 @@ class DatasetCoordinates(AbstractCoordinates):
                                   for k, v in self._data.variables.items()
                                   if k in self._names))
 
+    def __getitem__(self, key):
+        if key in self._data.data_vars:
+            raise KeyError(key)
+        return self._data[key]
+
     def to_dataset(self):
         """Convert these coordinates into a new Dataset
         """
@@ -156,14 +182,13 @@ class DatasetCoordinates(AbstractCoordinates):
 
         # check for inconsistent state *before* modifying anything in-place
         dims = calculate_dimensions(variables)
+        new_coord_names = set(coords)
         for dim, size in dims.items():
-            if dim not in variables:
-                variables[dim] = default_index_coordinate(dim, size)
-
-        updated_coord_names = set(coords) | set(dims)
+            if dim in variables:
+                new_coord_names.add(dim)
 
         self._data._variables = variables
-        self._data._coord_names.update(updated_coord_names)
+        self._data._coord_names.update(new_coord_names)
         self._data._dims = dict(dims)
 
     def __delitem__(self, key):
@@ -172,13 +197,17 @@ class DatasetCoordinates(AbstractCoordinates):
         else:
             raise KeyError(key)
 
+    def _ipython_key_completions_(self):
+        """Provide method for the key-autocompletions in IPython. """
+        return [key for key in self._data._ipython_key_completions_()
+                if key not in self._data.data_vars]
+
 
 class DataArrayCoordinates(AbstractCoordinates):
     """Dictionary like container for DataArray coordinates.
 
     Essentially an OrderedDict with keys given by the array's
-    dimensions and the values given by the corresponding xarray.Coordinate
-    objects.
+    dimensions and the values given by corresponding DataArray objects.
     """
     def __init__(self, dataarray):
         self._data = dataarray
@@ -187,11 +216,14 @@ class DataArrayCoordinates(AbstractCoordinates):
     def _names(self):
         return set(self._data._coords)
 
+    def __getitem__(self, key):
+        return self._data._getitem_coord(key)
+
     def _update_coords(self, coords):
         from .dataset import calculate_dimensions
 
         dims = calculate_dimensions(coords)
-        if set(dims) != set(self.dims):
+        if not set(dims) <= set(self.dims):
             raise ValueError('cannot add coordinates with new dimensions to '
                              'a DataArray')
         self._data._coords = coords
@@ -210,43 +242,62 @@ class DataArrayCoordinates(AbstractCoordinates):
         return self._to_dataset()
 
     def __delitem__(self, key):
-        if key in self.dims:
-            raise ValueError('cannot delete a coordinate corresponding to a '
-                             'DataArray dimension')
         del self._data._coords[key]
 
+    def _ipython_key_completions_(self):
+        """Provide method for the key-autocompletions in IPython. """
+        return self._data._ipython_key_completions_()
+
+
+class LevelCoordinatesSource(object):
+    """Iterator for MultiIndex level coordinates.
+
+    Used for attribute style lookup with AttrAccessMixin. Not returned directly
+    by any public methods.
+    """
+    def __init__(self, data_object):
+        self._data = data_object
+
+    def __getitem__(self, key):
+        # not necessary -- everything here can already be found in coords.
+        raise KeyError
+
+    def __iter__(self):
+        return iter(self._data._level_coords)
+
 
 class Indexes(Mapping, formatting.ReprMixin):
     """Ordered Mapping[str, pandas.Index] for xarray objects.
     """
-    def __init__(self, variables, dims):
+    def __init__(self, variables, sizes):
         """Not for public consumption.
 
         Arguments
         ---------
-        variables : OrderedDict
+        variables : OrderedDict[Any, Variable]
             Reference to OrderedDict holding variable objects. Should be the
             same dictionary used by the source object.
-        dims : sequence or mapping
-            Should be the same dimensions used by the source object.
+        sizes : OrderedDict[Any, int]
+            Map from dimension names to sizes.
         """
         self._variables = variables
-        self._dims = dims
+        self._sizes = sizes
 
     def __iter__(self):
-        return iter(self._dims)
+        for key in self._sizes:
+            if key in self._variables:
+                yield key
 
     def __len__(self):
-        return len(self._dims)
+        return sum(key in self._variables for key in self._sizes)
 
     def __contains__(self, key):
-        return key in self._dims
+        return key in self._sizes and key in self._variables
 
     def __getitem__(self, key):
-        if key in self:
-            return self._variables[key].to_index()
-        else:
+        if key not in self._sizes:
             raise KeyError(key)
+        return self._variables[key].to_index()
 
     def __unicode__(self):
         return formatting.indexes_repr(self)
