@@ -8,13 +8,13 @@ import functools
 import itertools
 import re
 import warnings
-from collections import Mapping, MutableMapping, Iterable
+from collections import Mapping, MutableMapping, MutableSet, Iterable
 
 import numpy as np
 import pandas as pd
 
-from . import ops
-from .pycompat import iteritems, OrderedDict, basestring, bytes_type
+from .pycompat import (iteritems, OrderedDict, basestring, bytes_type,
+                       dask_array_type)
 
 
 def alias_message(old_name, new_name):
@@ -98,8 +98,10 @@ def equivalent(first, second):
     """Compare two objects for equivalence (identity or equality), using
     array_equiv if either object is an ndarray
     """
+    # TODO: refactor to avoid circular import
+    from . import duck_array_ops
     if isinstance(first, np.ndarray) or isinstance(second, np.ndarray):
-        return ops.array_equiv(first, second)
+        return duck_array_ops.array_equiv(first, second)
     else:
         return ((first is second) or
                 (first == second) or
@@ -187,7 +189,7 @@ def is_scalar(value):
     return (
         getattr(value, 'ndim', None) == 0 or
         isinstance(value, (basestring, bytes_type)) or not
-        isinstance(value, Iterable))
+        isinstance(value, (Iterable, ) + dask_array_type))
 
 
 def is_valid_numpy_dtype(dtype):
@@ -378,6 +380,43 @@ class ChainMap(MutableMapping, SingleSlotPickleMixin):
         raise len(iter(self))
 
 
+class OrderedSet(MutableSet):
+    """A simple ordered set.
+
+    The API matches the builtin set, but it preserves insertion order of
+    elements, like an OrderedDict.
+    """
+    def __init__(self, values=None):
+        self._ordered_dict = OrderedDict()
+        if values is not None:
+            self |= values
+
+    # Required methods for MutableSet
+
+    def __contains__(self, value):
+        return value in self._ordered_dict
+
+    def __iter__(self):
+        return iter(self._ordered_dict)
+
+    def __len__(self):
+        return len(self._ordered_dict)
+
+    def add(self, value):
+        self._ordered_dict[value] = None
+
+    def discard(self, value):
+        del self._ordered_dict[value]
+
+    # Additional methods
+
+    def update(self, values):
+        self |= values
+
+    def __repr__(self):
+        return '%s(%r)' % (type(self).__name__, list(self))
+
+
 class NdimSizeLenMixin(object):
     """Mixin class that extends a class that defines a ``shape`` property to
     one that also defines ``ndim``, ``size`` and ``__len__``.
@@ -398,12 +437,7 @@ class NdimSizeLenMixin(object):
             raise TypeError('len() of unsized object')
 
 
-class DunderArrayMixin(object):
-    def __array__(self, dtype=None):
-        return np.asarray(self[...], dtype=dtype)
-
-
-class NDArrayMixin(NdimSizeLenMixin, DunderArrayMixin):
+class NDArrayMixin(NdimSizeLenMixin):
     """Mixin class for making wrappers of N-dimensional arrays that conform to
     the ndarray interface required for the data argument to Variable objects.
 
@@ -425,6 +459,16 @@ class NDArrayMixin(NdimSizeLenMixin, DunderArrayMixin):
         return '%s(array=%r)' % (type(self).__name__, self.array)
 
 
+class ReprObject(object):
+    """Object that prints as the given value, for use with sentinel values."""
+
+    def __init__(self, value):  # type: str
+        self._value = value
+
+    def __repr__(self):
+        return self._value
+
+
 @contextlib.contextmanager
 def close_on_error(f):
     """Context manager to ensure that a file opened by xarray is closed if an
